@@ -1,6 +1,6 @@
 /* gtk-playlist.c
 
-   Copyright (C) 2002 Bastien Nocera
+   Copyright (C) 2002, 2003 Bastien Nocera
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -52,8 +52,9 @@ struct GtkPlaylistPrivate
 	/* This is a scratch list for when we're removing files */
 	GList *list;
 
-	/* This is the current path for the file selector */
+	/* These is the current paths for the file selectors */
 	char *path;
+	char *save_path;
 
 	/* Repeat mode */
 	gboolean repeat;
@@ -242,6 +243,103 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 	return mimetype;
 }
 
+static gboolean
+write_string (GnomeVFSHandle *handle, const char *buf)
+{
+	GnomeVFSResult res;
+	GnomeVFSFileSize written;
+	int len;
+
+	len = strlen (buf);
+	res = gnome_vfs_write (handle, buf, len, &written);
+	if (res != GNOME_VFS_OK || written < len)
+	{
+		//FIXME
+		g_message ("write_string: %s", gnome_vfs_result_to_string (res));
+		gnome_vfs_close (handle);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+void
+gtk_playlist_save_current_playlist (GtkPlaylist *playlist, const char *output)
+{
+	GnomeVFSHandle *handle;
+	GnomeVFSResult res;
+	int num_entries, i, written, len;
+	char *buf;
+	gboolean success;
+
+	num_entries = gtk_tree_model_iter_n_children
+		(playlist->_priv->model, NULL);
+	res = gnome_vfs_open (&handle, output, GNOME_VFS_OPEN_WRITE);
+	if (res == GNOME_VFS_ERROR_NOT_FOUND)
+	{
+		res = gnome_vfs_create (&handle, output,
+				GNOME_VFS_OPEN_WRITE, FALSE,
+				GNOME_VFS_PERM_USER_WRITE
+				| GNOME_VFS_PERM_USER_READ
+				| GNOME_VFS_PERM_GROUP_READ);
+	}
+
+	if (res != GNOME_VFS_OK)
+	{
+		g_message ("gtk_playlist_save_current_playlist: %s", gnome_vfs_result_to_string (res));
+		//FIXME
+		return;
+	}
+
+	buf = g_strdup ("[playlist]\n");
+	success = write_string (handle, buf);
+	g_free (buf);
+	if (success == FALSE)
+		return;
+
+	buf = g_strdup_printf ("numberofentries=%d\n", num_entries);
+	success = write_string (handle, buf);
+	g_free (buf);
+	if (success == FALSE)
+		return;
+
+	for (i = 1; i <= num_entries; i++)
+	{
+		GtkTreeIter iter;
+		char *path, *mrl, *title;
+
+		path = g_strdup_printf ("%d", i - 1);
+		gtk_tree_model_get_iter_from_string (playlist->_priv->model,
+				&iter, path);
+		g_free (path);
+
+		gtk_tree_model_get (playlist->_priv->model,
+				&iter,
+				URI_COL, &mrl,
+				FILENAME_COL, &title,
+				-1);
+
+		buf = g_strdup_printf ("file%d=%s\n", i, mrl);
+		success = write_string (handle, buf);
+		g_free (buf);
+		g_free (mrl);
+		if (success == FALSE)
+		{
+			g_free (title);
+			return;
+		}
+
+		buf = g_strdup_printf ("title%d=%s\n", i, title);
+		success = write_string (handle, buf);
+		g_free (buf);
+		g_free (title);
+		if (success == FALSE)
+			return;
+	}
+
+	gnome_vfs_close (handle);
+}
+
 static void
 gtk_tree_selection_has_selected_foreach (GtkTreeModel *model,
 		GtkTreePath *path, GtkTreeIter *iter, gpointer user_data)
@@ -391,9 +489,8 @@ drop_cb (GtkWidget     *widget,
 	 GtkSelectionData   *data, 
 	 guint               info, 
 	 guint               time, 
-	 gpointer            user_data)
+	 GtkPlaylist        *playlist)
 {
-	GtkPlaylist *playlist = GTK_PLAYLIST (user_data);
 	GList *list, *p, *file_list;
 
 	list = gnome_vfs_uri_list_parse (data->data);
@@ -454,9 +551,8 @@ drop_cb (GtkWidget     *widget,
 }
 
 static void
-selection_changed (GtkTreeSelection *treeselection, gpointer user_data)
+selection_changed (GtkTreeSelection *treeselection, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = GTK_PLAYLIST (user_data);
 	GtkWidget *remove_button, *up_button, *down_button;
 	gboolean sensitivity;
 
@@ -497,9 +593,8 @@ update_current_from_playlist (GtkPlaylist *playlist)
 }
 
 static void
-gtk_playlist_add_files (GtkWidget *widget, gpointer user_data)
+gtk_playlist_add_files (GtkWidget *widget, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = (GtkPlaylist *)user_data;
 	GtkWidget *fs;
 	int response;
 
@@ -509,14 +604,12 @@ gtk_playlist_add_files (GtkWidget *widget, gpointer user_data)
 	{
 		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
 				playlist->_priv->path);
-		g_free (playlist->_priv->path);
-		playlist->_priv->path = NULL;
 	}
 	response = gtk_dialog_run (GTK_DIALOG (fs));
 	gtk_widget_hide (fs);
 	while (gtk_events_pending())
 		gtk_main_iteration();
-	
+
 	if (response == GTK_RESPONSE_OK)
 	{
 		char **filenames;
@@ -530,6 +623,7 @@ gtk_playlist_add_files (GtkWidget *widget, gpointer user_data)
 			char *tmp;
 
 			tmp = g_path_get_dirname (filenames[0]);
+			g_free (playlist->_priv->path);
 			playlist->_priv->path = g_strconcat (tmp,
 					G_DIR_SEPARATOR_S, NULL);
 			g_free (tmp);
@@ -559,12 +653,12 @@ gtk_playlist_foreach_selected (GtkTreeModel *model, GtkTreePath *path,
 }
 
 static void
-gtk_playlist_remove_files (GtkWidget *widget, gpointer user_data)
+gtk_playlist_remove_files (GtkWidget *widget, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = (GtkPlaylist *)user_data;
 	GtkTreeSelection *selection;
-	GtkTreeRowReference *ref = NULL;
+	GtkTreeRowReference *ref;
 	gboolean is_selected = FALSE;
+	int next_pos;
 
 	selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW (playlist->_priv->treeview));
@@ -579,11 +673,20 @@ gtk_playlist_remove_files (GtkWidget *widget, gpointer user_data)
 	 * reference to it, TreeIter and TreePath don't allow that */
 	if (playlist->_priv->current != NULL)
 	{
+		int *indices;
+
 		ref = gtk_tree_row_reference_new (playlist->_priv->model,
 				playlist->_priv->current);
 		is_selected = gtk_tree_selection_path_is_selected (selection,
 				playlist->_priv->current);
+
+		indices = gtk_tree_path_get_indices (playlist->_priv->current);
+		next_pos = indices[0];
+
 		gtk_tree_path_free (playlist->_priv->current);
+	} else {
+		ref = NULL;
+		next_pos = -1;
 	}
 
 	/* We destroy the items, one-by-one from the list built above */
@@ -604,12 +707,33 @@ gtk_playlist_remove_files (GtkWidget *widget, gpointer user_data)
 		playlist->_priv->list = g_list_remove (playlist->_priv->list,
 				playlist->_priv->list->data);
 	}
+	g_list_free (playlist->_priv->list);
 	playlist->_priv->list = NULL;
 
 	if (is_selected == TRUE)
 	{
 		/* The current item was removed from the playlist */
-		playlist->_priv->current = NULL;
+		if (next_pos != -1)
+		{
+			char *str;
+			GtkTreeIter iter;
+			GtkTreePath *cur;
+
+			str = g_strdup_printf ("%d", next_pos);
+			cur = gtk_tree_path_new_from_string (str);
+
+			if (gtk_tree_model_get_iter (playlist->_priv->model,
+						&iter, cur) == FALSE)
+			{
+				playlist->_priv->current = NULL;
+				gtk_tree_path_free (cur);
+			} else {
+				playlist->_priv->current = cur;
+			}
+		} else {
+			playlist->_priv->current = NULL;
+		}
+
 		g_signal_emit (G_OBJECT (playlist),
 				gtk_playlist_table_signals[CURRENT_REMOVED], 0,
 				NULL);
@@ -627,6 +751,198 @@ gtk_playlist_remove_files (GtkWidget *widget, gpointer user_data)
 	}
 }
 
+static void
+gtk_playlist_save_files (GtkWidget *widget, GtkPlaylist *playlist)
+{
+	GtkWidget *fs;
+	int response;
+
+	fs = gtk_file_selection_new (_("Save playlist"));
+	if (playlist->_priv->save_path != NULL)
+	{
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+				playlist->_priv->save_path);
+	}
+
+	response = gtk_dialog_run (GTK_DIALOG (fs));
+	gtk_widget_hide (fs);
+	while (gtk_events_pending())
+		gtk_main_iteration();
+
+	if (response == GTK_RESPONSE_OK)
+	{
+		const char *filename;
+
+		filename = gtk_file_selection_get_filename
+			(GTK_FILE_SELECTION (fs));
+		if (filename != NULL)
+		{
+			char *tmp;
+
+			tmp = g_path_get_dirname (filename);
+			g_free (playlist->_priv->save_path);
+			playlist->_priv->save_path = g_strconcat (tmp,
+					G_DIR_SEPARATOR_S, NULL);
+			g_free (tmp);
+		}
+
+		if (g_file_test (filename, G_FILE_TEST_EXISTS) == TRUE)
+		{
+			GtkWidget *dialog;
+
+			dialog = gtk_message_dialog_new
+				(GTK_WINDOW (playlist),
+				 GTK_DIALOG_MODAL,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_NONE,
+				 _("A file named '%s' already exists.\nAre you sure you want to overwrite it?"),
+				 filename);
+			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					NULL);
+
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			if (response != GTK_RESPONSE_ACCEPT)
+			{
+				gtk_widget_destroy (fs);
+				return;
+			}
+		}
+
+		gtk_playlist_save_current_playlist (playlist, filename);
+	}
+
+	gtk_widget_destroy (fs);
+}
+
+static void
+gtk_playlist_move_files (GtkPlaylist *playlist, gboolean direction_up)
+{
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkListStore *store;
+	GtkTreeIter iter;
+	GtkTreeRowReference *current;
+	GList *paths, *refs, *l;
+	int pos;
+
+	selection = gtk_tree_view_get_selection
+		(GTK_TREE_VIEW (playlist->_priv->treeview));
+	if (selection == NULL)
+		return;
+
+	model = gtk_tree_view_get_model
+		(GTK_TREE_VIEW (playlist->_priv->treeview));
+	store = GTK_LIST_STORE (model);
+	pos = -2;
+	refs = NULL;
+
+	if (playlist->_priv->current != NULL)
+	{
+		current = gtk_tree_row_reference_new (model,
+				playlist->_priv->current);
+	} else {
+		current = NULL;
+	}
+
+	/* Build a list of tree references */
+	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
+	for (l = paths; l != NULL; l = l->next)
+	{
+		GtkTreePath *path = l->data;
+		int cur_pos, *indices;
+
+		refs = g_list_prepend (refs,
+				gtk_tree_row_reference_new (model, path));
+		indices = gtk_tree_path_get_indices (path);
+		cur_pos = indices[0];
+		if (pos == -2)
+		{
+			pos = cur_pos;
+		} else {
+			if (direction_up == FALSE)
+				pos = MAX (cur_pos, pos);
+			else
+				pos = MIN (cur_pos, pos);
+		}
+	}
+	g_list_foreach (paths, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (paths);
+
+	refs = g_list_reverse (refs);
+
+	if (direction_up == FALSE)
+		pos = pos + 2;
+	else
+		pos = pos - 2;
+
+	for (l = refs; l != NULL; l = l->next)
+	{
+		GtkTreeIter *position, current;
+		GtkTreeRowReference *ref = l->data;
+		GtkTreePath *path;
+
+		if (pos < 0)
+		{
+			position = NULL;
+		} else {
+			char *str;
+
+			str = g_strdup_printf ("%d", pos);
+			if (gtk_tree_model_get_iter_from_string (model,
+					&iter, str))
+				position = &iter;
+			else
+				position = NULL;
+
+			g_free (str);
+		}
+
+		path = gtk_tree_row_reference_get_path (ref);
+		gtk_tree_model_get_iter (model, &current, path);
+		gtk_tree_path_free (path);
+
+		if (direction_up == FALSE)
+		{
+			pos--;
+			gtk_list_store_move_before (store, &current, position);
+		} else {
+			gtk_list_store_move_after (store, &current, position);
+			pos++;
+		}
+	}
+
+	g_list_foreach (refs, (GFunc) gtk_tree_row_reference_free, NULL);
+	g_list_free (refs);
+
+	/* Update the current path */
+	if (current != NULL)
+	{
+		gtk_tree_path_free (playlist->_priv->current);
+		playlist->_priv->current = gtk_tree_row_reference_get_path
+			(current);
+		gtk_tree_row_reference_free (current);
+	}
+
+	g_signal_emit (G_OBJECT (playlist),
+			gtk_playlist_table_signals[CHANGED], 0,
+			NULL);
+}
+
+static void
+gtk_playlist_up_files (GtkWidget *widget, GtkPlaylist *playlist)
+{
+	gtk_playlist_move_files (playlist, TRUE);
+}
+
+static void
+gtk_playlist_down_files (GtkWidget *widget, GtkPlaylist *playlist)
+{
+	gtk_playlist_move_files (playlist, FALSE);
+}
+
 static void
 init_columns (GtkTreeView *treeview)
 {
@@ -651,10 +967,8 @@ init_columns (GtkTreeView *treeview)
 
 static void
 treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
-		GtkTreeViewColumn *arg2, gpointer user_data)
+		GtkTreeViewColumn *arg2, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = (GtkPlaylist *)user_data;
-
 	if (gtk_tree_path_equals (arg1, playlist->_priv->current) == TRUE)
 		return;
 
@@ -707,9 +1021,8 @@ init_treeview (GtkWidget *treeview, GtkPlaylist *playlist)
 }
 
 static void
-repeat_button_toggled (GtkToggleButton *togglebutton, gpointer user_data)
+repeat_button_toggled (GtkToggleButton *togglebutton, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = (GtkPlaylist *)user_data;
 	gboolean repeat;
 
 	repeat = gtk_toggle_button_get_active (togglebutton);
@@ -724,9 +1037,8 @@ repeat_button_toggled (GtkToggleButton *togglebutton, gpointer user_data)
 
 static void
 update_repeat_cb (GConfClient *client, guint cnxn_id,
-		GConfEntry *entry, gpointer user_data)
+		GConfEntry *entry, GtkPlaylist *playlist)
 {
-	GtkPlaylist *playlist = (GtkPlaylist *)user_data;
 	GtkWidget *button;
 	gboolean repeat;
 
@@ -765,7 +1077,8 @@ init_config (GtkPlaylist *playlist)
 	gconf_client_add_dir (playlist->_priv->gc, GCONF_PREFIX,
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	gconf_client_notify_add (playlist->_priv->gc, GCONF_PREFIX"/repeat",
-			update_repeat_cb, playlist, NULL, NULL);
+			(GConfClientNotifyFunc) update_repeat_cb,
+			playlist, NULL, NULL);
 	g_signal_connect (G_OBJECT (button), "toggled",
 			G_CALLBACK (repeat_button_toggled),
 			(gpointer) playlist);
@@ -897,19 +1210,31 @@ gtk_playlist_new (const char *glade_filename, GdkPixbuf *playing_pix)
 	gtk_window_set_default_size (GTK_WINDOW (playlist),
 			300, 375);
 	g_signal_connect_object (GTK_OBJECT (playlist), "response",
-			GTK_SIGNAL_FUNC (gtk_widget_hide), 
+			G_CALLBACK (gtk_widget_hide), 
 			GTK_WIDGET (playlist),
 			0);
 
 	/* Connect the buttons */
 	item = glade_xml_get_widget (playlist->_priv->xml, "add_button");
 	g_signal_connect (GTK_OBJECT (item), "clicked",
-			GTK_SIGNAL_FUNC (gtk_playlist_add_files),
-			(gpointer) playlist);
+			G_CALLBACK (gtk_playlist_add_files),
+			playlist);
 	item = glade_xml_get_widget (playlist->_priv->xml, "remove_button");
 	g_signal_connect (GTK_OBJECT (item), "clicked",
-			GTK_SIGNAL_FUNC (gtk_playlist_remove_files),
-			(gpointer) playlist);
+			G_CALLBACK (gtk_playlist_remove_files),
+			playlist);
+	item = glade_xml_get_widget (playlist->_priv->xml, "save_button");
+	g_signal_connect (GTK_OBJECT (item), "clicked",
+			G_CALLBACK (gtk_playlist_save_files),
+			playlist);
+	item = glade_xml_get_widget (playlist->_priv->xml, "up_button");
+	g_signal_connect (GTK_OBJECT (item), "clicked",
+			G_CALLBACK (gtk_playlist_up_files),
+			playlist);
+	item = glade_xml_get_widget (playlist->_priv->xml, "down_button");
+	g_signal_connect (GTK_OBJECT (item), "clicked",
+			G_CALLBACK (gtk_playlist_down_files),
+			playlist);
 
 	container = glade_xml_get_widget (playlist->_priv->xml, "vbox4");
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (playlist)->vbox),
@@ -1465,9 +1790,10 @@ char
 	if (update_current_from_playlist (playlist) == FALSE)
 		return NULL;
 
-	gtk_tree_model_get_iter (playlist->_priv->model,
-			&iter,
-			playlist->_priv->current);
+	if (gtk_tree_model_get_iter (playlist->_priv->model, &iter,
+			playlist->_priv->current) == FALSE)
+		return NULL;
+
 	gtk_tree_model_get (playlist->_priv->model,
 			&iter,
 			URI_COL, &path,
