@@ -11,7 +11,10 @@
 #include <X11/X.h>
 #include <X11/Xlib.h>
 
-void scrsaver_init(Display *display);
-void scrsaver_enable(Display *display);
-void scrsaver_disable(Display *display);
+typedef struct ScreenSaver ScreenSaver;
+
+ScreenSaver *scrsaver_new(Display *display);
+void scrsaver_enable(ScreenSaver *scr);
+void scrsaver_disable(ScreenSaver *scr);
+void scrsaver_free(ScreenSaver *scr);
 
