@@ -2,6 +2,7 @@
 #include "config.h"
 
 #include <gnome.h>
+#include <string.h>
 #include "bacon-video-widget.h"
 
 /* #define THUMB_DEBUG */
@@ -165,6 +166,8 @@ int main (int argc, char *argv[])
 	GdkPixbuf *pixbuf;
 	int i;
 
+	g_thread_init (NULL);
+
 	gtk_init (&argc, &argv);
 
 	if (argc != 3)
